/*
* $Id: //depot/prod/kamino/main/util/viewServerInfo/src/ViewServerInfo.cpp#3 $
*
* Copyright (c) 2008 NetApp, Inc.
* All rights reserved.
*/

#include "ViewServerInfo.h"

/*
* Class:     com_netapp_kamino_server_ViewServerInfo
* Method:    validateCredentials
* Signature: (Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
*/
JNIEXPORT jint JNICALL Java_com_netapp_kamino_server_view_ViewServerInfo_validateCredentials
(JNIEnv *env, jclass joClass, jstring jhost, jstring jntLogon, jstring jpassword) {
	char* host = (char*)env->GetStringUTFChars(jhost, NULL);
	char* ntLogon = (char*)env->GetStringUTFChars(jntLogon, NULL);
	char* password = (char*)env->GetStringUTFChars(jpassword, NULL);

	int response = ViewServerCmnd::ValidateCredentials(host, ntLogon, password);

	env->ReleaseStringUTFChars(jhost, host);
	env->ReleaseStringUTFChars(jntLogon, ntLogon);
	env->ReleaseStringUTFChars(jpassword, password);

	return response;
}

/*
* Class:     com_netapp_kamino_server_ViewServerInfo
* Method:    getVirtualCenterGUID
* Signature: (Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;
*/
JNIEXPORT jobject JNICALL Java_com_netapp_kamino_server_view_ViewServerInfo_getVirtualCenterGUID
(JNIEnv *env, jclass joClass, jstring jhost, jstring jntLogon, jstring jpassword, jobjectArray jaddresses) {
	char* host = (char*)env->GetStringUTFChars(jhost, NULL);
	char* ntLogon = (char*)env->GetStringUTFChars(jntLogon, NULL);
	char* password = (char*)env->GetStringUTFChars(jpassword, NULL);

	// converts a passed-in jobjectarray of String to c++ strings
	jsize addressCount = env->GetArrayLength(jaddresses);
	vector<string> addresses;
	for(jsize i = 0; i < addressCount; i++) {
		jstring jaddress = (jstring)env->GetObjectArrayElement(jaddresses, i);
		char* address = (char*)env->GetStringUTFChars(jaddress, NULL);
		addresses.push_back(address);
        env->ReleaseStringUTFChars(jaddress, address);
	}

	char dn[512];
	int response = ViewServerCmnd::GetVirtualCenterGUID(host, ntLogon, password, addresses, dn);

	env->ReleaseStringUTFChars(jhost, host);
	env->ReleaseStringUTFChars(jntLogon, ntLogon);
	env->ReleaseStringUTFChars(jpassword, password);

	return getViewServerRet1(env, response, dn);
}

/*
* Class:     com_netapp_kamino_server_ViewServerInfo
* Method:    getPools
* Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)[Ljava/lang/String;
*/
JNIEXPORT jobject JNICALL Java_com_netapp_kamino_server_view_ViewServerInfo_getPools
(JNIEnv *env, jclass joClass, jstring jhost, jstring jntLogon, jstring jpassword, jboolean manualPoolsOnly) {
	char* host = (char*)env->GetStringUTFChars(jhost, NULL);
	char* ntLogon = (char*)env->GetStringUTFChars(jntLogon, NULL);
	char* password = (char*)env->GetStringUTFChars(jpassword, NULL);

	vector<ViewPool*> pools;
	int response = ViewServerCmnd::GetPools(host, ntLogon, password, manualPoolsOnly ? true : false, pools);

	jobject ret = getViewServerRet2(env, response, pools);

	// clean up
	env->ReleaseStringUTFChars(jhost, host);
	env->ReleaseStringUTFChars(jntLogon, ntLogon);
	env->ReleaseStringUTFChars(jpassword, password);

	vector<ViewPool*>::iterator itr;
	for(itr = pools.begin(); itr != pools.end(); itr++) {
		delete *itr;
	}

	return ret;
}

/*
 * Class:     com_netapp_kamino_server_view_ViewServerInfo
 * Method:    ImportPool
 * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;IILjava/lang/String;[Lcom/netapp/kamino/server/view/DesktopSource;)V
 *
 * Param: jaccessMode - the access mode of the pool (dedicated or floating)
 * Param: jpoolType - either new or existing pool
 */
JNIEXPORT jint JNICALL Java_com_netapp_kamino_server_view_ViewServerInfo_importPool
(JNIEnv *env, jclass joClass, jstring jhost, jstring jntLogon, jstring jpassword, jobjectArray jaddresses, jint jaccessMode, jint jpoolType,
	jobjectArray jdesktopSources) {
	char* host = (char*)env->GetStringUTFChars(jhost, NULL);
	char* ntLogon = (char*)env->GetStringUTFChars(jntLogon, NULL);
	char* password = (char*)env->GetStringUTFChars(jpassword, NULL);
	int accessMode = jaccessMode;
	int poolType = jpoolType;

	// converts a passed-in jobjectarray of String to c++ strings
	jsize addressCount = env->GetArrayLength(jaddresses);
	vector<string> addresses;
	for(jsize i = 0; i < addressCount; i++) {
		jstring jaddress = (jstring)env->GetObjectArrayElement(jaddresses, i);
		char* address = (char*)env->GetStringUTFChars(jaddress, NULL);
		addresses.push_back(address);
        env->ReleaseStringUTFChars(jaddress, address);
	}

	// converts a passed-in jobjectarray of DesktopSources to a c++ vector
	jsize sourceCount = env->GetArrayLength(jdesktopSources);
	map<string, vector<ViewDesktopSource*> > desktopSourceMap;
	for(jsize i = 0; i < sourceCount; i++) {
		jobject jdesktopSource = (jobject)env->GetObjectArrayElement(jdesktopSources, i);
		ViewDesktopSource *desktopSource = new ViewDesktopSource();
		JDesktopSourceToCDesktopSource(env, jdesktopSource, desktopSource);
		desktopSourceMap[desktopSource->getPool()].push_back(desktopSource);
	}

	int response = ViewServerCmnd::ImportPool(host, ntLogon, password, addresses, accessMode, poolType, desktopSourceMap);

	env->ReleaseStringUTFChars(jhost, host);
	env->ReleaseStringUTFChars(jntLogon, ntLogon);
	env->ReleaseStringUTFChars(jpassword, password);

	map<string, vector<ViewDesktopSource*> >::iterator poolItr;
	for(poolItr = desktopSourceMap.begin(); poolItr != desktopSourceMap.end(); poolItr++) {
		vector<ViewDesktopSource*>::iterator desktopSourceItr;
		for(desktopSourceItr = (*poolItr).second.begin(); desktopSourceItr != (*poolItr).second.end(); desktopSourceItr++) {
			delete *desktopSourceItr;
		}
	}

	return response;
}

void JDesktopSourceToCDesktopSource(JNIEnv* env, jobject jdesktopSource, ViewDesktopSource* desktopSource) {
	jclass clazz = env->GetObjectClass(jdesktopSource);

	jfieldID fieldID = env->GetFieldID(clazz, "name", "Ljava/lang/String;");
	jstring value = (jstring)env->GetObjectField(jdesktopSource, fieldID);
	desktopSource->setName(env->GetStringUTFChars(value, NULL));

	fieldID = env->GetFieldID(clazz, "moid", "Ljava/lang/String;");
	value = (jstring)env->GetObjectField(jdesktopSource, fieldID);
	desktopSource->setMoid(env->GetStringUTFChars(value, NULL));

	fieldID = env->GetFieldID(clazz, "domain", "Ljava/lang/String;");
	value = (jstring)env->GetObjectField(jdesktopSource, fieldID);
	desktopSource->setDomain(env->GetStringUTFChars(value, NULL));

	fieldID = env->GetFieldID(clazz, "datacenter", "Ljava/lang/String;");
	value = (jstring)env->GetObjectField(jdesktopSource, fieldID);
	desktopSource->setDatacenter(env->GetStringUTFChars(value, NULL));

	fieldID = env->GetFieldID(clazz, "pool", "Ljava/lang/String;");
	value = (jstring)env->GetObjectField(jdesktopSource, fieldID);
	desktopSource->setPool(env->GetStringUTFChars(value, NULL));
}

jobject getViewServerRet1(JNIEnv* env, int response, char* dn) {
    jclass clazz = env->FindClass("com/netapp/kamino/server/view/ViewServerRet");
    jmethodID constructor = env->GetMethodID(clazz, "<init>", "()V");
    jobject jret = env->NewObject(clazz, constructor);

    // Set each field in ViewServerRet one at a time
    jfieldID fid = env->GetFieldID(clazz, "returnCode", "I");
    env->SetIntField(jret, fid, response);

    fid = env->GetFieldID(clazz, "vcdn", "Ljava/lang/String;");
    env->SetObjectField(jret, fid, env->NewStringUTF(dn));

	return jret;
}

jobject getViewServerRet2(JNIEnv* env, int response, vector<ViewPool*> &pools) {
	// create a jobjectarray of ViewPools
	jclass clazz = env->FindClass("com/netapp/kamino/server/view/ViewPool");
    jmethodID constructor = env->GetMethodID(clazz, "<init>", "()V");
	
	jobjectArray jpools = (jobjectArray)env->NewObjectArray((jsize)pools.size(), clazz, NULL);
	for(int i = 0; i < (env->GetArrayLength(jpools)); i++) {
		jobject jpool = env->NewObject(clazz, constructor);
		jfieldID fid = env->GetFieldID(clazz, "name", "Ljava/lang/String;");
		env->SetObjectField(jpool, fid, env->NewStringUTF(pools[i]->getName().c_str()));
		fid = env->GetFieldID(clazz, "vcdn", "Ljava/lang/String;");
		env->SetObjectField(jpool, fid, env->NewStringUTF(pools[i]->getVcdn().c_str()));
		env->SetObjectArrayElement(jpools, i, jpool);
	}

	// now create the response object
    clazz = env->FindClass("com/netapp/kamino/server/view/ViewServerRet");
    constructor = env->GetMethodID(clazz, "<init>", "()V");
    jobject jret = env->NewObject(clazz, constructor);

    // add the return code
    jfieldID fid = env->GetFieldID(clazz, "returnCode", "I");
    env->SetIntField(jret, fid, response);

	// add the pools
    fid = env->GetFieldID(clazz, "pools", "[Lcom/netapp/kamino/server/view/ViewPool;");
    env->SetObjectField(jret, fid, jpools);

	return jret;
}

